"use strict";

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var os = require('os');

var ms = require('ms');

var chalk = require('chalk');

var _require = require('./utils.js'),
    inspectOpts = _require.inspectOpts,
    saveOpts = _require.saveOpts,
    inspectNamespaces = _require.inspectNamespaces,
    selectColor = _require.selectColor,
    levelColors = _require.levelColors;

var LogBase = require('./LogBase.js');

var wrapConsole = require('./wrapConsole.js');

var env = process.env.NODE_ENV || 'development';
var isDevEnv = /^dev/.test(env); // anything which starts with dev is seen as development env

var EXIT_EVENTS = ['unhandledRejection', 'uncaughtException'];
/**
 * global log options
 */

var options = {
  level: undefined,
  namespaces: undefined,
  json: !isDevEnv,
  // log in json format
  serverinfo: !isDevEnv,
  // append server information
  hideDate: isDevEnv,
  // do not hide date from output
  colors: isDevEnv,
  // apply colors
  stream: process.stderr,
  // output stream
  spaces: null,
  // pretty print JSON
  splitLine: isDevEnv
};
/**
 * creates a new logger
 * @constructor
 * @param {String} name - namespace of Logger
 */

function Log(name, opts) {
  if (!(this instanceof Log)) return new Log(name, opts);
  Object.assign(options, inspectOpts(process.env), inspectNamespaces(process.env));
  LogBase.call(this, name, Object.assign({}, options, opts));

  var colorFn = function colorFn(n) {
    return chalk.hex(n);
  };

  this.color = selectColor(name, colorFn);
  this.levColors = levelColors(colorFn);

  if (!this.opts.json) {
    this._log = this._logOneLine;
  }
}

Object.setPrototypeOf(Log.prototype, LogBase.prototype);
Object.assign(Log.prototype, {
  /**
   * render string to output stream
   * @public
   * @param {String} str - string to render
   * @param {String} level - level of log line (might be used for custom Logger which uses different streams per level)
   * @return {String}
   */
  render: function render(str) {
    str += '\n';
    this.opts.stream.write(str);
    return str;
  },

  /**
   * format object to json
   * @private
   */
  _log: function _log(level, args) {
    var _this = this;

    this._diff();

    var o = this._formatJson(level, args);

    var str = this.formatter.format(o)[0];
    /* c8 ignore next */
    // can't cover with test as underlying tty is unknown

    if (this.opts.colors) {
      // this is slow...
      str = str.replace(/"level":\s?"([^"]+)"/, function (m, level) {
        return _this._color(m, _this.levColors[level], true);
      }).replace(/"name":\s?"[^"]+"/, function (m) {
        return _this._color(m, _this.color, true);
      });
    }

    return this.render(str, level);
  },

  /**
   * debug like output if `this.opts.json === false`
   * @private
   */
  _logOneLine: function _logOneLine(level, args) {
    this._diff();

    var str = this._formatArgs(level, args);

    return this.render(str, level);
  },

  /**
   * format arguments to debug like string
   * @private
   * @param {String} level
   * @param {Array} args - formatter arguments - first arg should contain "%" formatter directives
   * @return {String} formatted String
   */
  _formatArgs: function _formatArgs(level, _args) {
    var _this$formatter,
        _this2 = this;

    this.formatter.noQuotes = true;

    var args = (_this$formatter = this.formatter).format.apply(_this$formatter, _toConsumableArray(_args));

    var msg = args.shift(); // if there are still unformatted args push through formatter and append to msg

    if (args.length) {
      msg += ' ' + args.map(function (arg) {
        return _this2.formatter.format('%O', arg);
      }).join(' ');
    }

    this.formatter.noQuotes = false;

    var prefix = '  ' + this._color(level, this.levColors[level], true) + ' ' + this._color(this.name, this.color, true);

    var str = [prefix, this.opts.hideDate ? '' : new Date().toISOString(), !this.opts.splitLine ? msg.replace(/[\r\n]/g, '\\n') : msg.split(/\\n|\n/).join('\n' + prefix + ' '), this._color('+' + ms(this.diff), this.color), this.opts.serverinfo ? os.hostname() + ' ' + process.pid : undefined].filter(function (f) {
      return f;
    }).join(' ');
    return str;
  },

  /**
   * Add colors, style to string
   * @private
   */
  _color: function _color(str, color, isBold) {
    return !this.opts.colors ? str : isBold ? color.bold(str) : color(str);
  },

  /**
   * append server info to json object - used by `_formatJson`
   * @private
   */
  _serverinfo: function _serverinfo(o) {
    if (this.opts.serverinfo) {
      Object.assign(o, {
        hostname: os.hostname(),
        pid: process.pid
      });
    }
  }
});
/**
* Apply (and get) global options
* @param {object} [opts] - changed options
* @return {object} global options
*/

Log.options = function (opts) {
  if (!opts) return Object.assign({}, options);
  Object.assign(options, opts);
  return options;
};
/**
 * save options in `process.env`
 */


Log.save = function () {
  Log.reset();
  saveOpts(process.env, options);
};
/**
 * reset saved options
 */


Log.reset = function () {
  Object.keys(process.env).forEach(function (key) {
    if (/^(DEBUG|DEBUG_.*)$/.test(key)) {
      delete process.env[key];
    }
  });
};

Log.isDevEnv = isDevEnv;
/**
 * wrap console logging functions like
 * console.log, console.info, console.warn, console.error
 * @param {string} [name='console']
 * @param {object} opts - see Log.options
 * @param {string} [opts.level4log='log'] - log level for console.log
 * @return {function} unwrap function
 */

Log.wrapConsole = function () {
  var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'console';
  var opts = arguments.length > 1 ? arguments[1] : undefined;
  var log = new Log(name, opts);
  return wrapConsole(log, opts);
};
/**
 * log exit events like 'unhandledRejection', 'uncaughtException'
 * and then let the process die
 * @param {string} [name='exit']
 * @param {object} opts - see Log.options
 * @param {boolean} [opts.code=1] - set exit code; code=0 will prevent triggering exit
 * @param {boolean} [opts.gracefulExit=false] - uses process.exitCode to avoid forceful exit with process.exit()
 */


Log.handleExitEvents = function handleExitEvents() {
  var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'exit';
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _opts$code = opts.code,
      code = _opts$code === void 0 ? 1 : _opts$code,
      gracefulExit = opts.gracefulExit,
      _opts = _objectWithoutProperties(opts, ["code", "gracefulExit"]);

  var log = new Log(name, _opts);
  EXIT_EVENTS.forEach(function (ev) {
    process.on(ev, function (err) {
      log.fatal(err);
      /* c8 ignore next 7 */

      if (code) {
        if (gracefulExit) {
          process.exitCode = code; // let die...
        } else {
          setTimeout(function () {
            return process.exit(code);
          }, 5);
        }
      }
    });
  });
};

module.exports = Log;